<?php

/**
 * @file
 * Search page callback file.
 */

/**
 * Callback function for Qwant search page.
 */
function qwantsearch_display_search_page() {
  // Looks like our title callback doesn't work so...
  drupal_set_title(variable_get('qwantsearch_search_page_title'));

  $qwant = new Qwantsearch(variable_get('qwantsearch_http_token', ''), variable_get('qwantsearch_partner_id', ''));

  $params = drupal_get_query_parameters();
  $qwant_params = array();

  $params_mapping = array(
    'search' => 'q',
    'offset' => 'offset',
    'rows' => 'count',
    'order' => 'f',
  );

  foreach ($params_mapping as $query_param => $qwant_param) {
    if (isset($params[$query_param])) {
      $qwant_params[$qwant_param] = $params[$query_param];
    }
  }

  // Force number of items to value configured in BO.
  $qwant_params['count'] = variable_get('qwantsearch_nb_items_displayed', QWANTSEARCH_DEFAULT_NB_ITEMS);

  $query = $qwant->makeQuery($qwant_params);
  $response = $qwant->parseResponse($query);

  if (!$qwant->isSuccess($response)) {
    watchdog('qwantsearch', 'An error occurred during qwant query execution. Status: !status . Error: !error', array(
      '!status' => $response['status'],
      '!error' => $response['error'],
    ));
    return t('An error occurred. Please check that Qwant services are running and contact an administrator.');
  }

  $content = qwantsearch_prepare_response_content($response);

  return $content;
}

/**
 * Prepares a string to be displayed containing formatted search results.
 *
 * @param array $response
 *   Array returned by Qwantsearch->parseResponse.
 *
 * @return null|string
 *   Rendered html using templates or "no result" string.
 *
 * @throws \Exception
 */
function qwantsearch_prepare_response_content(array $response) {
  $results = $response['data']['results'];
  $renderable_results = array();
  $nb_elements = count($results);

  if ($nb_elements == 0) {
    $content = filter_xss_admin(variable_get('qwantsearch_no_result', ''));
    return $content;
  }

  foreach ($results as $index => $result) {
    $renderable_result = array(
      '#theme' => 'qwantsearch_search_result',
      '#title' => strip_tags(html_entity_decode($result['title'], ENT_QUOTES)),
      '#url' => $result['url'],
      '#snippet' => strip_tags(html_entity_decode($result['desc'], ENT_QUOTES)),
      '#date' => $result['date'],
      '#picture' => qwantsearch_generate_result_image($result['media']),
      '#row_attributes' => array(
        'class' => array(
          'qwant-search-result',
        ),
      ),
    );
    // Allow module to change variables given the qwant result.
    drupal_alter('qwantsearch_search_result', $renderable_result, $result);

    $renderable_result['#row_attributes']['class'][] = ($index % 2 == 0) ? 'odd' : 'even';
    if ($index == 0) {
      $renderable_result['#row_attributes']['class'][] = 'first';
    }
    if ($index == $nb_elements - 1) {
      $renderable_result['#row_attributes']['class'][] = 'last';
    }
    $renderable_results[] = $renderable_result;
  }

  drupal_add_css(drupal_get_path('module', 'qwantsearch') . '/css/qwantsearch.css');

  return theme('qwantsearch_results_page', array('results' => $renderable_results));
}

/**
 * Generates a thumbnail for search result using module imagecache_external.
 *
 * @param array $medias
 *   Medias returned in Qwant json response.
 *
 * @return bool|string
 *   HTML for picture using configured image style.
 */
function qwantsearch_generate_result_image(array $medias) {
  $picture = '';
  foreach ($medias as $media) {
    if ($media['type'] == 'image') {
      $image_variables = array(
        'width' => $media['width'],
        'height' => $media['height'],
        'path' => $media['url'],
        'style_name' => variable_get('qwantsearch_result_image_style', 'thumbnail'),
      );

      drupal_alter('qwantsearch_result_thumbnail', $image_variables);
      $picture = theme('imagecache_external', $image_variables);
    }
  }

  return $picture;
}
